; To remove an export, add a semicolon at the beginning of the line to comment it out.

EXPORTS
ilActiveImage
ilActiveLayer
ilActiveMipmap
ilApplyProfile
ilApplyPal
ilBindImage
ilBlit
ilClearColour
ilClearImage
ilCloneCurImage
ilCompressFunc
ilConvertImage
ilConvertPal
ilCopyImage
ilCopyPixels
ilCreateSubImage
ilDefaultImage
ilDeleteImages
ilDetermineType ; added 2003-09-17
ilDisable
ilEnable
ilFormatFunc
ilGenImages
ilGetAlpha
ilGetBoolean
ilGetBooleanv
ilGetBppFormat
ilGetBppPal
ilGetBppType
ilGetClear
ilGetCurImage
ilGetCurName
ilGetData
ilGetDXTCData
ilGetError
ilGetInteger
ilGetIntegerv
ilGetLumpPos
ilGetPalBaseType
ilGetPalette
ilGetString
ilGetTypeBpc
ilHint
ilInit
ilIsDisabled
ilIsEnabled
ilIsImage
ilIsValid
ilIsValidF
ilIsValidL
ilKeyColour
ilLoad
ilLoadF
ilLoadImage
ilLoadL
ilLoadPal
ilNextPower2
ilOriginFunc
ilOverlayImage
ilPopAttrib
ilPushAttrib
ilRegisterFormat
ilRegisterLoad
ilRegisterMipNum
ilRegisterNumImages
ilRegisterOrigin
ilRegisterPal
ilRegisterSave
ilRegisterType
ilRemoveLoad
ilRemoveSave
ilReplaceCurImage
ilResetMemory
ilResetRead
ilResetWrite
ilSave
ilSaveF
ilSaveImage
ilSaveL
ilSavePal
ilSaveData
ilSetData
ilSetDuration
ilSetError
ilSetInteger
ilSetMemory
ilSetPal
ilSetRead
ilSetPixels
ilSetString
ilSetWrite
ilShutDown
ilTexImage
ilTypeFromExt
ilTypeFunc
ilLoadData
ilLoadDataF
ilLoadDataL
ilLoadFromJpegStruct
ilSaveFromJpegStruct

; Internal but exported functions
iBindImageTemp
ilClearImage_
ilConvertBuffer
ilCloseImage
ilClosePal
iConvertImage
iConvertPal
iCopyPal
ilCopyImageAttr
ilCopyImage_
iGetFlipped
ilIsValidPal
ilNewImage
ilResizeImage
ilSetCurImage
ilTexImage_
ilTexSubImage_
ialloc
ifree


; Export the API of the included libjpeg so that applications
; can use both libraries without linking to two separate copies.
;jpeg_std_error
;jpeg_CreateCompress
;jpeg_CreateDecompress
;jpeg_destroy_compress
;jpeg_destroy_decompress
;jpeg_stdio_dest
;jpeg_stdio_src
;jpeg_set_defaults
;jpeg_set_colorspace
;jpeg_default_colorspace
;jpeg_set_quality
;jpeg_set_linear_quality
;jpeg_add_quant_table
;jpeg_quality_scaling
;jpeg_simple_progression
;jpeg_suppress_tables
;jpeg_alloc_quant_table
;jpeg_alloc_huff_table
;jpeg_start_compress
;jpeg_write_scanlines
;jpeg_finish_compress
;jpeg_write_raw_data
;jpeg_write_marker
;jpeg_write_m_header
;jpeg_write_m_byte
;jpeg_write_tables
;jpeg_read_header
;jpeg_start_decompress
;jpeg_read_scanlines
;jpeg_finish_decompress
;jpeg_read_raw_data
;jpeg_has_multiple_scans
;jpeg_start_output
;jpeg_finish_output
;jpeg_input_complete
;jpeg_new_colormap
;jpeg_consume_input
;jpeg_calc_output_dimensions
;jpeg_save_markers
;jpeg_set_marker_processor
;jpeg_read_coefficients
;jpeg_write_coefficients
;jpeg_copy_critical_parameters
;jpeg_abort_compress
;jpeg_abort_decompress
;jpeg_abort
;jpeg_destroy
;jpeg_resync_to_restart

;jpeg_copy_key_exif_parameters
;jpeg_extract_exif_entries
;jpeg_init_exif_field
;jpeg_build_exif_marker
;jpeg_init_exif_field_table
