; To remove an export, add a semicolon at the beginning of the line to comment it out.

EXPORTS
ilutConvertToHBitmap
;ilutConvertToSDLSurface
;ilutSDLSurfaceFromBitmap
;ilutSDLSurfaceLoadImage
;ilutD3D8LoadSurface
;ilutD3D8MipFunc
;ilutD3D8Texture
;ilutD3D8TexFromFile
;ilutD3D8VolumeTexture
;ilutD3D8VolTexFromFile
;ilutD3D8TexFromFileInMemory
;ilutD3D8VolTexFromFileInMemory
;ilutD3D8TexFromFileHandle
;ilutD3D8VolTexFromFileHandle
;ilutD3D8TexFromResource
;ilutD3D8VolTexFromResource
;ilutD3D9LoadSurface
;ilutD3D9MipFunc
;ilutD3D9Texture
;ilutD3D9TexFromFile
;ilutD3D9VolumeTexture
;ilutD3D9VolTexFromFile
;ilutD3D9TexFromFileInMemory
;ilutD3D9VolTexFromFileInMemory
;ilutD3D9TexFromFileHandle
;ilutD3D9VolTexFromFileHandle
;ilutD3D9TexFromResource
;ilutD3D9VolTexFromResource
ilutDisable
ilutEnable
ilutFreePaddedData
ilutGetBmpInfo
ilutGetBoolean
ilutGetBooleanv
ilutGetHPal
ilutGetInteger
ilutGetIntegerv
ilutGetPaddedData
ilutGetString
ilutGetWinClipboard
ilutInit
ilutIsDisabled
ilutIsEnabled
ilutLoadResource
ilutGLBindMipmaps
ilutGLBindTexImage
ilutGLBuildMipmaps
ilutGLLoadImage
ilutGLSaveImage
ilutGLScreen
ilutGLScreenie
ilutGLSetTex
ilutGLTexImage
ilutPopAttrib
ilutPushAttrib
ilutRenderer
ilutSetHBitmap
ilutSetHPal
ilutSetInteger
ilutSetWinClipboard
ilutWinLoadImage
ilutWinLoadUrl
ilutWinPrint
ilutWinSaveImage
