; To remove an export, add a semicolon at the beginning of the line to comment it out.

EXPORTS
iluAlienify
iluBlurAvg
iluBlurGaussian
iluBuildMipmaps
iluColoursUsed
iluCompareImage
iluContrast
iluCrop
iluDeleteImage
iluEdgeDetectE
iluEdgeDetectP
iluEdgeDetectS
iluEmboss
iluEnlargeCanvas
iluEnlargeImage
iluEqualize
iluErrorString
iluFlipImage
iluGenImage
iluGetImageInfo
iluGetString
iluGammaCorrect
iluGetInteger
iluGetIntegerv
iluImageParameter
iluInit
iluInvertAlpha
iluLoadImage
iluMirror
iluNegative
iluNoisify
iluPixelize
iluRegionfv
iluRegioniv
iluReplaceColour
iluRotate
iluRotate3D
iluSaturate1f
iluSaturate4f
iluScaleColours
iluScale
iluSharpen
iluSwapColours
iluWave


; Internal but exported functions
iluRotate_
iluRotate3D_
iluScale_
