#ifndef __CONFIG_H__
#define __CONFIG_H__
@TOP@

/* Supported images formats (il) */

#undef IL_NO_BMP
#undef IL_NO_CUT
#undef IL_NO_CHEAD
#undef IL_NO_DCX
#undef IL_NO_DDS
#undef IL_NO_DOOM
#undef IL_NO_GIF
#undef IL_NO_HDR
#undef IL_NO_ICO
#undef IL_NO_JPG
#undef IL_NO_LCMS
#undef IL_NO_LIF
#undef IL_NO_MDL
#undef IL_NO_MNG
#undef IL_NO_PCD
#undef IL_NO_PCX
#undef IL_NO_PIC
#undef IL_NO_PIX
#undef IL_NO_PNG
#undef IL_NO_PNM
#undef IL_NO_PSD
#undef IL_NO_PSP
#undef IL_NO_PXR
#undef IL_NO_RAW
#undef IL_NO_SGI
#undef IL_NO_TGA
#undef IL_NO_TIF
#undef IL_NO_WAL
#undef IL_NO_XPM
#undef IL_USE_JPEGLIB_UNMODIFIED

/* Supported api (ilut) */

#undef ILUT_USE_ALLEGRO
#undef ILUT_USE_DIRECTX8
#undef ILUT_USE_OPENGL
#undef ILUT_USE_SDL
#undef ILUT_USE_WIN32

@BOTTOM@
#endif /* __CONFIG_H__ */
