
EXPORTS
    mng_initialize          @1
    mng_reset               @2
    mng_cleanup             @3
    mng_read                @4
    mng_write               @5
    mng_create              @6
    mng_readdisplay         @7
    mng_display             @8
    mng_display_resume      @9
    mng_display_freeze      @10
    mng_display_reset       @11
    mng_display_goframe     @12
    mng_display_golayer     @13
    mng_display_gotime      @14
    mng_getlasterror        @15

    mng_setcb_memalloc      @31
    mng_setcb_memfree       @32
    mng_setcb_openstream    @33
    mng_setcb_closestream   @34
    mng_setcb_readdata      @35
    mng_setcb_writedata     @36
    mng_setcb_errorproc     @37
;    mng_setcb_traceproc     @38
    mng_setcb_processheader @39
    mng_setcb_processtext   @40
    mng_setcb_getcanvasline @41
    mng_setcb_getbkgdline   @42
    mng_setcb_refresh       @43
    mng_setcb_gettickcount  @44
    mng_setcb_settimer      @45
;    mng_setcb_processgamma  @46
;    mng_setcb_processchroma @47
;    mng_setcb_processsrgb   @48
;    mng_setcb_processiccp   @49
;    mng_setcb_processarow   @50

    mng_getcb_memalloc      @61
    mng_getcb_memfree       @62
    mng_getcb_openstream    @63
    mng_getcb_closestream   @64
    mng_getcb_readdata      @65
    mng_getcb_writedata     @66
    mng_getcb_errorproc     @67
;    mng_getcb_traceproc     @68
    mng_getcb_processheader @69
    mng_getcb_processtext   @70
    mng_getcb_getcanvasline @71
    mng_getcb_getbkgdline   @72
    mng_getcb_refresh       @73
    mng_getcb_gettickcount  @74
    mng_getcb_settimer      @75
;    mng_getcb_processgamma  @76
;    mng_getcb_processchroma @77
;    mng_getcb_processsrgb   @78
;    mng_getcb_processiccp   @79
;    mng_getcb_processarow   @80

    mng_set_userdata        @91
    mng_set_canvasstyle     @92
    mng_set_bkgdstyle       @93
    mng_set_bgcolor         @94
    mng_set_storechunks     @95
    mng_set_viewgamma       @96
    mng_set_displaygamma    @97
    mng_set_dfltimggamma    @98
    mng_set_viewgammaint    @100
    mng_set_displaygammaint @101
    mng_set_dfltimggammaint @102
    mng_set_srgb            @103
    mng_set_outputprofile   @104
    mng_set_srgbprofile     @105
    mng_set_maxcanvaswidth  @106
    mng_set_maxcanvasheight @107
    mng_set_maxcanvassize   @108

    mng_get_userdata        @121
    mng_get_sigtype         @122
    mng_get_imagetype       @123
    mng_get_imagewidth      @124
    mng_get_imageheight     @125
    mng_get_ticks           @126
    mng_get_framecount      @127
    mng_get_layercount      @128
    mng_get_playtime        @129
    mng_get_simplicity      @130
    mng_get_canvasstyle     @131
    mng_get_bkgdstyle       @132
    mng_get_bgcolor         @133
    mng_get_storechunks     @134
    mng_get_viewgamma       @135
    mng_get_displaygamma    @136
    mng_get_dfltimggamma    @137
    mng_get_viewgammaint    @138
    mng_get_displaygammaint @139
    mng_get_dfltimggammaint @140
    mng_get_srgb            @141
    mng_get_maxcanvaswidth  @142
    mng_get_maxcanvasheight @143